Imports System.Windows
Imports System.Windows.Navigation
Imports System.Windows.Threading
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports System.Device.Location
Imports System.Net.NetworkInformation
Imports System.Net
Imports System.IO

Namespace SeaVan
	Public Delegate Sub AllDataCompleteEvent()

	Partial Public Class App
		Inherits Application

		#Region "Fields"
		Private phoneApplicationInitialized As Boolean = False
		Private Shared ReadOnly lockObject As New Object

		' The refresh interval is: canada ~5min, usa ~2min.
		Private Const TIMER_INTERVAL As Double = 180
		Private Const CANADA_URL As String = "http://apps.cbp.gov/bwt/bwt.xml"
		Private Const USA_URL As String = "http://wsdot.wa.gov/traffic/rssfeeds/CanadianBorderTrafficData/Default.aspx"

		Friend Const ERROR_MESSAGE_LOCATION_RACE As String = "Cannot get your start location - please try again in a little while"
		Friend Const ERROR_MESSAGE_NETWORK As String = "Cannot get border data - please check your network connection, and try again later"
		Friend Const ERROR_MESSAGE_DATA_CA As String = "Cannot get Canadian border data - please try again in a little while"
		Friend Const ERROR_MESSAGE_DATA_US As String = "Cannot get US border data - please try again in a little while"
		Friend Const ERROR_MESSAGE_LOCATION As String = "Your last known location is stale - directions will be approximate"
		Friend Const ERROR_MESSAGE_GENERAL As String = "Sorry, we've hit a problem - please restart the app"
		Friend Const ERROR_MESSAGE_DIRECTIONS As String = "Cannot get directions at the moment - please try again in a little while"
		Friend Const ERROR_MESSAGE_MARKETPLACE As String = "Cannot connect to marketplace at the moment - please try again later"

		Private Shared geoWatcher As GeoCoordinateWatcher
		Private Const GEOWATCHER_THRESHOLD As Double = 1000
		Private Const GEOWATCHER_TIMEOUT As Double = 10
		Private Shared isDataCompleteUs As Boolean
		Private Shared isDataCompleteCa As Boolean

		Public Shared Event AllDataComplete As AllDataCompleteEvent
		#End Region

		#Region "Properties"
        Private _RootFrame As PhoneApplicationFrame
        Public Property RootFrame As PhoneApplicationFrame
            Get
                Return _RootFrame
            End Get
            Private Set(ByVal value As PhoneApplicationFrame)
                _RootFrame = value
            End Set
        End Property

        Private Shared _crossings As BorderCrossings
        Public Shared ReadOnly Property Crossings As BorderCrossings
            Get
                SyncLock lockObject
                    If _crossings Is Nothing Then
                        _crossings = New BorderCrossings
                    End If
                End SyncLock
                Return _crossings
            End Get
        End Property

		Private Shared settings As AppSettings
		Public Shared ReadOnly Property AppSettings As AppSettings
			Get
				SyncLock lockObject
					If settings Is Nothing Then
						settings = New AppSettings
					End If
				End SyncLock
				Return settings
			End Get
		End Property

        Private Shared _timer As DispatcherTimer
        Public Shared ReadOnly Property Timer As DispatcherTimer
            Get
                SyncLock lockObject
                    If _timer Is Nothing Then
                        _timer = New DispatcherTimer
                        _timer.Interval = TimeSpan.FromSeconds(TIMER_INTERVAL)
                    End If
                End SyncLock
                Return _timer
            End Get
        End Property

        Private Shared _lastKnownPosition As GeoCoordinate
        Public Shared ReadOnly Property LastKnownPosition As GeoCoordinate
            Get
                Return _lastKnownPosition
            End Get
        End Property

        Private Shared _isGeoWatcherRunning As Boolean
        Public Shared ReadOnly Property IsGeoWatcherRunning As Boolean
            Get
                Return _isGeoWatcherRunning
            End Get
        End Property
#End Region

#Region "GeoCoordinateWatcher"
        Public Shared Sub StartGeoWatcher()
            If Not _isGeoWatcherRunning Then
                If AppSettings.LocationSetting Then
                    SyncLock lockObject
                        If geoWatcher Is Nothing Then
                            geoWatcher = New GeoCoordinateWatcher(GeoPositionAccuracy.High)
                            geoWatcher.MovementThreshold = GEOWATCHER_THRESHOLD
                        End If
                        AddHandler geoWatcher.PositionChanged, AddressOf geoWatcher_PositionChanged
                        geoWatcher.Start()
                        _isGeoWatcherRunning = (geoWatcher.Status = GeoPositionStatus.Initializing OrElse geoWatcher.Status = GeoPositionStatus.Ready)
                    End SyncLock
                End If
            End If
        End Sub

        Private Shared Sub geoWatcher_PositionChanged(ByVal sender As Object, ByVal e As GeoPositionChangedEventArgs(Of GeoCoordinate))
            If e.Position IsNot Nothing AndAlso e.Position.Location IsNot Nothing Then
                _lastKnownPosition = e.Position.Location
            End If
        End Sub

        Public Shared Sub StopGeoWatcher()
            If geoWatcher IsNot Nothing Then
                RemoveHandler geoWatcher.PositionChanged, AddressOf geoWatcher_PositionChanged
                geoWatcher.Stop()
            End If
        End Sub
#End Region

		#Region "Phone application initialization"
		Private Sub InitializePhoneApplication()
			If phoneApplicationInitialized Then
				Exit Sub
			End If

			RootFrame = New PhoneApplicationFrame
			AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
			AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed
			phoneApplicationInitialized = True
		End Sub

		Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
			If RootVisual IsNot RootFrame Then
				RootVisual = RootFrame
			End If
			RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
		End Sub
		#End Region

		#Region "Lifetime events"
		Public Sub New()
            AddHandler UnhandledException, AddressOf Application_UnhandledException

            'If System.Diagnostics.Debugger.IsAttached Then
            '    Application.Current.Host.Settings.EnableFrameRateCounter = True
            '    'Application.Current.Host.Settings.EnableRedrawRegions = True
            '    'Application.Current.Host.Settings.EnableCacheVisualization = True
            'End If

			InitializeComponent()
			InitializePhoneApplication()
		End Sub

		Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
		End Sub

		Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
		End Sub

		Private Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
		End Sub

		Private Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)
		End Sub
		#End Region

		#Region "Exceptions"
		Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
			If Debugger.IsAttached Then
				Debugger.Break()
			Else
				ShowErrorMessage(ERROR_MESSAGE_GENERAL)
				e.Handled = True
			End If
		End Sub

		Private Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs)
			If Debugger.IsAttached Then
				Debugger.Break()
			Else
				ShowErrorMessage(ERROR_MESSAGE_GENERAL)
				e.Handled = True
			End If
		End Sub

		Friend Shared Sub ShowErrorMessage(ByVal message As String)
			If Debugger.IsAttached Then
				Debug.WriteLine(message)
			Else
				If Deployment.Current.Dispatcher.CheckAccess Then
					MessageBox.Show(message)
				Else
					Deployment.Current.Dispatcher.BeginInvoke(Function() MessageBox.Show(message))
				End If
			End If
		End Sub
		#End Region

		#Region "FetchWaitTimes"
		Public Shared Sub FetchWaitTimes()
			StartGeoWatcher()

			If Not NetworkInterface.GetIsNetworkAvailable Then
				ShowErrorMessage(ERROR_MESSAGE_NETWORK)
				Exit Sub
			End If

			isDataCompleteUs = False
			isDataCompleteCa = False
			Dim webClientUsa As New WebClient
			AddHandler webClientUsa.OpenReadCompleted, AddressOf webClientUs_OpenReadCompleted

			' We have to append a unique (unused) querystring to force the WebClient to re-fetch instead of using the cache.
			Try
				webClientUsa.OpenReadAsync(New Uri(USA_URL & "?" & Guid.NewGuid.ToString))
			Catch e1 As WebException
				ShowErrorMessage(ERROR_MESSAGE_DATA_US)
				isDataCompleteUs = True
				If isDataCompleteUs AndAlso isDataCompleteCa Then
					RaiseEvent AllDataComplete()
				End If
			End Try

			Dim webClientCa As New WebClient
			AddHandler webClientCa.OpenReadCompleted, AddressOf webClientCa_OpenReadCompleted

			Try
				webClientCa.OpenReadAsync(New Uri(CANADA_URL & "?" & Guid.NewGuid.ToString))
			Catch e2 As WebException
				ShowErrorMessage(ERROR_MESSAGE_DATA_CA)
				isDataCompleteCa = True
				If isDataCompleteUs AndAlso isDataCompleteCa Then
					RaiseEvent AllDataComplete()
				End If
			End Try
		End Sub
		#End Region

		#Region "webClientUs_OpenReadCompleted"
		Private Shared Sub webClientUs_OpenReadCompleted(ByVal sender As Object, ByVal e As OpenReadCompletedEventArgs)
			If e.Error IsNot Nothing Then
				ShowErrorMessage(ERROR_MESSAGE_DATA_US)
				Exit Sub
			End If

			Using result As Stream = e.Result
				Try
					CrossingXmlParser.ParseXmlUs(result)
				Catch ex As Exception
					Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted - CrossingXmlParser.ParseXmlUs: {0}", ex.Message))
					ShowErrorMessage(ERROR_MESSAGE_DATA_US)
				Finally
					isDataCompleteUs = True
					If isDataCompleteUs AndAlso isDataCompleteCa Then
						RaiseEvent AllDataComplete()
					End If
				End Try
			End Using
		End Sub
		#End Region

		#Region "webClientCa_OpenReadCompleted"
		Private Shared Sub webClientCa_OpenReadCompleted(ByVal sender As Object, ByVal e As OpenReadCompletedEventArgs)
			If e.Error IsNot Nothing Then
				ShowErrorMessage(ERROR_MESSAGE_DATA_CA)
				Exit Sub
			End If

			Using result As Stream = e.Result
				Try
					CrossingXmlParser.ParseXmlCa(result)
				Catch e1 As Exception
					ShowErrorMessage(ERROR_MESSAGE_DATA_CA)
				Finally
					isDataCompleteCa = True
					If isDataCompleteUs AndAlso isDataCompleteCa Then
						RaiseEvent AllDataComplete()
					End If
				End Try
			End Using
		End Sub
		#End Region

	End Class
End Namespace
