Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Microsoft.Phone.Tasks

Namespace SeaVan
	Partial Public Class MainPage
		Inherits PhoneApplicationPage

		#Region "Fields"
		Private Const BORDER_LATITUDE As Double = 49.0022
		Private progress As ProgressIndicator
		#End Region

		#Region "Init"
		Public Sub New()
			InitializeComponent()

			appBarRefresh = CType(ApplicationBar.Buttons(1), ApplicationBarIconButton)

			DataContext = App.Crossings
			CrossingsList.ItemsSource = App.Crossings.Crossings

			progress = New ProgressIndicator
			progress.IsIndeterminate = True
			progress.Text = "fetching latest data..."

			AddHandler Loaded, AddressOf MainPage_Loaded
			AddHandler App.AllDataComplete, AddressOf App_AllDataComplete
		End Sub

		Private Sub App_AllDataComplete()
			If Not App.AppSettings.AutoRefreshSetting Then
				appBarRefresh.IsEnabled = True
			End If
			progress.IsVisible = False
		End Sub

		' Loaded happens after OnNavigatedTo, which is after the first screen is shown,
		' so we do the expensive operation here to improve startup time. We also
		' unhook the event after we're done - because we only need to do this once at
		' startup - subsequently, we'll refresh during OnNavigatedTo.
		Private Sub MainPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			FetchWaitTimes()
			RemoveHandler Loaded, AddressOf MainPage_Loaded
		End Sub

		Private Sub FetchWaitTimes()
			' Start the progress indicator here, and stop it when both async web calls, and the results parsing are complete.
			progress.IsVisible = True
			App.FetchWaitTimes()
		End Sub
		#End Region

		#Region "Navigation"
		Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
			SystemTray.ProgressIndicator = progress

			If App.AppSettings.AutoRefreshSetting Then
				AddHandler App.Timer.Tick, AddressOf timer_Tick
				App.Timer.Start()

				If e.NavigationMode = NavigationMode.Back Then
					FetchWaitTimes()
				End If
			End If

			appBarRefresh.IsEnabled = Not App.AppSettings.AutoRefreshSetting

			App.StartGeoWatcher()
		End Sub

		Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
			RemoveHandler App.Timer.Tick, AddressOf timer_Tick
			App.Timer.Stop()
			App.StopGeoWatcher()
		End Sub

		Private Sub timer_Tick(ByVal sender As Object, ByVal e As EventArgs)
			FetchWaitTimes()
		End Sub
		#End Region

		#Region "AppBar Clicks"
		Private Sub appBarRefresh_Click(ByVal sender As Object, ByVal e As EventArgs)
			appBarRefresh.IsEnabled = False
			FetchWaitTimes()
		End Sub

		Private Sub appBarSettings_Click(ByVal sender As Object, ByVal e As EventArgs)
			NavigationService.Navigate(New Uri("/SettingsPage.xaml", UriKind.Relative))
		End Sub

		Private Sub appBarAbout_Click(ByVal sender As Object, ByVal e As EventArgs)
			NavigationService.Navigate(New Uri("/AboutPage.xaml", UriKind.Relative))
		End Sub
		#End Region

		#Region "CrossingsList_SelectionChanged"
		' We need to use a GeoCoordinateWatcher to figure out which side of the border we're on,
		' so that we can tell Bing to give us directions without crossing the border.
		Private Sub CrossingsList_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			If (Not App.AppSettings.LocationSetting) OrElse CrossingsList.SelectedIndex = -1 Then
				CrossingsList.SelectedIndex = -1
				Exit Sub
			End If

			If App.LastKnownPosition Is Nothing Then
				App.ShowErrorMessage(App.ERROR_MESSAGE_LOCATION_RACE)
				CrossingsList.SelectedIndex = -1
				Exit Sub
			End If

			If Not App.IsGeoWatcherRunning Then
				App.ShowErrorMessage(App.ERROR_MESSAGE_LOCATION)
			End If

			Dim crossing As BorderCrossing = CType(CrossingsList.SelectedItem, BorderCrossing)
			Dim directions As New BingMapsDirectionsTask

			' If we're below the 49th parallel, our target endpoint is on the US side of the border,
			' otherwise its on the Canadian side. 
			If App.LastKnownPosition.Latitude < BORDER_LATITUDE Then
				directions.End = New LabeledMapLocation(crossing.PortName, crossing.CoordinatesUs)
			Else
				directions.End = New LabeledMapLocation(crossing.PortName, crossing.CoordinatesCa)
			End If

			Try
				directions.Show()
			Catch
				App.ShowErrorMessage(App.ERROR_MESSAGE_DIRECTIONS)
			End Try

			CrossingsList.SelectedIndex = -1
		End Sub
		#End Region

	End Class
End Namespace
