Imports System.IO.IsolatedStorage

Namespace SeaVan
	Public Class AppSettings

		#Region "Fields"
		Private isolatedStore As IsolatedStorageSettings

		' The isolated storage key names of our settings
		Private Const LocationSettingKeyName As String = "LocationSetting"
		Private Const AutoRefreshSettingKeyName As String = "AutoRefreshSetting"

		' The default value of our settings
		Private Const LocationSettingDefault As Boolean = True
		Private Const AutoRefreshSettingDefault As Boolean = True
		#End Region

		#Region "Init"
		Public Sub New()
			Try
				isolatedStore = IsolatedStorageSettings.ApplicationSettings
			Catch ex As Exception
				Debug.WriteLine(ex.ToString)
			End Try
		End Sub

		' Update a setting value for our application. If the setting does not exist, then add the setting.
		Private Function AddOrUpdateValue(ByVal Key As String, ByVal value As Object) As Boolean
			Dim valueChanged As Boolean = False

			If isolatedStore.Contains(Key) Then
				If isolatedStore(Key) IsNot value Then
					isolatedStore(Key) = value
					valueChanged = True
				End If
			Else
				isolatedStore.Add(Key, value)
				valueChanged = True
			End If

			Return valueChanged
		End Function

		' Get the current value of the setting, or if it is not found, set the setting to the default setting.
		Private Function GetValueOrDefault(Of valueType)(ByVal Key As String, ByVal defaultValue As valueType) As valueType
			Dim value As valueType

			If isolatedStore.Contains(Key) Then
				value = CType(isolatedStore(Key), valueType)
			Else
				value = defaultValue
			End If

			Return value
		End Function

		Private Sub Save()
			isolatedStore.Save()
		End Sub
		#End Region

		#Region "Properties"
		Public Property LocationSetting As Boolean
			Get
				Return GetValueOrDefault(Of Boolean)(LocationSettingKeyName, LocationSettingDefault)
			End Get
			Set(ByVal value As Boolean)
				AddOrUpdateValue(LocationSettingKeyName, value)
				Save()
			End Set
		End Property

		Public Property AutoRefreshSetting As Boolean
			Get
				Return GetValueOrDefault(Of Boolean)(AutoRefreshSettingKeyName, AutoRefreshSettingDefault)
			End Get
			Set(ByVal value As Boolean)
				AddOrUpdateValue(AutoRefreshSettingKeyName, value)
				Save()
				If value Then
					App.Timer.Start()
				Else
					App.Timer.Stop()
				End If
			End Set
		End Property
		#End Region

	End Class
End Namespace
