Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Device.Location

Namespace SeaVan
	Public Class BorderCrossings
		Implements INotifyPropertyChanged

		#Region "INotifyPropertyChanged"
		Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

		Private Sub NotifyPropertyChanged(ByVal propertyName As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
		End Sub
		#End Region

		#Region "Properties"
        Private _crossings As New ObservableCollection(Of BorderCrossing)
		Public ReadOnly Property Crossings As ObservableCollection(Of BorderCrossing)
			Get
                Return _crossings
			End Get
		End Property

        Private _lastUpdateCa As String
		Public Property LastUpdateCa As String
			Get
                Return _lastUpdateCa
			End Get
			Set(ByVal value As String)
                If _lastUpdateCa <> value Then
                    _lastUpdateCa = value
                    NotifyPropertyChanged("LastUpdateCa")
                End If
			End Set
		End Property

        Private _lastUpdateUs As String
        Public Property LastUpdateUs As String
            Get
                Return _lastUpdateUs
            End Get
            Set(ByVal value As String)
                If _lastUpdateUs <> value Then
                    _lastUpdateUs = value
                    NotifyPropertyChanged("LastUpdateUs")
                End If
            End Set
        End Property
		#End Region

		#Region "Init"
		Public Sub New()
            _crossings.Add(New BorderCrossing With {.PortID = "300402", .PortName = "Peace Arch / I5", .CoordinatesCa = New GeoCoordinate(49.002315, -122.757277), .CoordinatesUs = New GeoCoordinate(49.001851, -122.755826), .ReadyDelayTimeUs = "-", .CommercialDelayTimeUs = "-", .FastDelayTimeUs = "-"})
            _crossings.Add(New BorderCrossing With {.PortID = "300401", .PortName = "Pacific Highway / SR543", .CoordinatesCa = New GeoCoordinate(49.003449, -122.735417), .CoordinatesUs = New GeoCoordinate(49.001084, -122.735245), .ReadyDelayTimeUs = "-"})
            _crossings.Add(New BorderCrossing With {.PortID = "302301", .PortName = "Aldegrove / Lynden", .CoordinatesCa = New GeoCoordinate(49.002333, -122.485322), .CoordinatesUs = New GeoCoordinate(49.002036, -122.484982), .NexusDelayTimeUs = "-", .ReadyDelayTimeUs = "-", .CommercialDelayTimeUs = "-", .FastDelayTimeUs = "-"})
            _crossings.Add(New BorderCrossing With {.PortID = "300901", .PortName = "Sumas / SR9", .CoordinatesCa = New GeoCoordinate(49.002731, -122.265971), .CoordinatesUs = New GeoCoordinate(49.002087, -122.265129), .NexusDelayTimeUs = "-", .ReadyDelayTimeUs = "-", .CommercialDelayTimeUs = "-", .FastDelayTimeUs = "-"})
		End Sub
		#End Region

		#Region "Indexer"
		Default Public ReadOnly Property Item(ByVal portID As String) As BorderCrossing
			Get
                Dim b = From c In _crossings Where c.PortID = portID Select c
                If b IsNot Nothing AndAlso b.Count > 0 Then
                    Return b.First
                End If
				Return Nothing
			End Get
		End Property
		#End Region

	End Class
End Namespace

